﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Shapes;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            this.sourceRectangle.MouseEnter += sourceRectangle_MouseEnter;
            this.sourceRectangle.MouseMove  += sourceRectangle_MouseMove;
            this.sourceRectangle.MouseLeave += sourceRectangle_MouseLeave;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 소스 사각형 마우스 진입시 처리하기 - sourceRectangle_MouseEnter(sender, e)

        /// <summary>
        /// 소스 사각형 마우스 진입시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void sourceRectangle_MouseEnter(object sender, MouseEventArgs e)
        {
            this.mousePointStackPanel.Visibility = Visibility.Visible;
        }

        #endregion
        #region 소스 사각형 마우스 이동시 처리하기 - sourceRectangle_MouseMove(sender, e)

        /// <summary>
        /// 소스 사각형 마우스 이동시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void sourceRectangle_MouseMove(object sender, MouseEventArgs e)
        {
            Rectangle rectangle = sender as Rectangle;

            Point mousePoint = e.GetPosition(rectangle);

            double rectangleWidth = rectangle.ActualWidth;

            double offset = mousePoint.X / rectangleWidth;

            Color color = GetColor(this.sourceBrush, offset);

            this.targetBrush.Color = color;

            this.targetColorTextBlock.Text = color.ToString();

            if(this.mousePointStackPanel.Visibility == Visibility.Visible)
            {
                this.mousePointTextBlock.Text = mousePoint.ToString();
            }
        }

        #endregion
        #region 소스 사각형 마우스 이탈시 처리하기 - sourceRectangle_MouseLeave(sender, e)

        /// <summary>
        /// 소스 사각형 마우스 이탈시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void sourceRectangle_MouseLeave(object sender, MouseEventArgs e)
        {
            this.mousePointStackPanel.Visibility = Visibility.Collapsed;

            this.targetBrush.Color = Colors.Transparent;

            this.targetColorTextBlock.Text = Colors.Transparent.ToString();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 색상 구하기 - GetColor(brush, offset)

        /// <summary>
        /// 색상 구하기
        /// </summary>
        /// <param name="brush">선형 그라디언트 브러시</param>
        /// <param name="offset">오프셋</param>
        /// <returns>색상</returns>
        private Color GetColor(LinearGradientBrush brush, double offset)
        {
            GradientStopCollection gradientStopCollection = brush.GradientStops;

            GradientStop[] gradientStopArray = gradientStopCollection.OrderBy(x => x.Offset).ToArray();

            if(offset <= 0)
            {
                return gradientStopArray[0].Color;
            }

            if(offset >= 1)
            {
                return gradientStopArray[gradientStopArray.Length - 1].Color;
            }

            GradientStop firstStop = gradientStopArray[0];
            GradientStop lastStop  = null;

            foreach(GradientStop stop in gradientStopArray)
            {
                if(stop.Offset >= offset)
                {
                    lastStop = stop;

                    break;
                }

                firstStop = stop;
            }

            offset = Math.Round((offset - firstStop.Offset) / (lastStop.Offset - firstStop.Offset), 2);

            byte alpha = (byte)((lastStop.Color.A - firstStop.Color.A) * offset + firstStop.Color.A);
            byte red   = (byte)((lastStop.Color.R - firstStop.Color.R) * offset + firstStop.Color.R);
            byte green = (byte)((lastStop.Color.G - firstStop.Color.G) * offset + firstStop.Color.G);
            byte blue  = (byte)((lastStop.Color.B - firstStop.Color.B) * offset + firstStop.Color.B);

            return Color.FromArgb(alpha, red, green, blue);
        }

        #endregion
    }
}